@@ -343,7 +343,9 @@ class ServerlessOperations(object):
     except api_exceptions.HttpNotFoundError:
       return None
 
-  def WaitService(self, operation_id):
+  def WaitService(
+      self, operation_id, service_ref, release_track=base.ReleaseTrack.GA
+  ):
     """Return the relevant Service from the server, or None if 404."""
     messages = self.messages_module
     project = properties.VALUES.core.project.Get(required=True)
@@ -360,10 +362,27 @@ class ServerlessOperations(object):
             self._client.projects_locations_services,
             self._client.projects_locations_operations,
         )
-        operation = waiter.PollUntilDone(poller, op_ref)
-        as_dict = encoding.MessageToPyValue(operation.response)
-        as_pb = encoding.PyValueToMessage(messages.Service, as_dict)
-        return service.Service(as_pb, self.messages_module)
+        if release_track == base.ReleaseTrack.ALPHA:
+          operation = poller.Poll(op_ref)
+          # operation.response will only be filled if the operation is done.
+          # if the opration isn't done, operation.metadata should have status
+          # information for the tracker.
+          if operation.response:
+            as_dict = encoding.MessageToPyValue(operation.response)
+            as_pb = encoding.PyValueToMessage(messages.Service, as_dict)
+            return service.Service(as_pb, self.messages_module)
+          elif operation.metadata:
+            as_dict = encoding.MessageToPyValue(operation.metadata)
+            as_pb = encoding.PyValueToMessage(messages.Service, as_dict)
+            return service.Service(as_pb, self.messages_module)
+          else:
+            return self.GetService(service_ref)
+        else:
+          operation = waiter.PollUntilDone(poller, op_ref)
+          as_dict = encoding.MessageToPyValue(operation.response)
+          as_pb = encoding.PyValueToMessage(messages.Service, as_dict)
+          return service.Service(as_pb, self.messages_module)
+
     except api_exceptions.InvalidDataFromServerError as e:
       serverless_exceptions.MaybeRaiseCustomFieldMismatch(e)
     except api_exceptions.HttpNotFoundError:
@@ -678,7 +697,13 @@ class ServerlessOperations(object):
       )
 
   def UpdateTraffic(
-      self, service_ref, config_changes, tracker, asyn, is_verbose
+      self,
+      service_ref,
+      config_changes,
+      tracker,
+      asyn,
+      is_verbose,
+      release_track,
   ):
     """Update traffic splits for service."""
     if tracker is None:
@@ -700,12 +725,15 @@ class ServerlessOperations(object):
     if not asyn:
       if updated_serv.conditions.IsReady():
         return updated_serv
-
-      getter = (
-          functools.partial(self.GetService, service_ref)
-          if updated_serv.operation_id is None or is_verbose
-          else functools.partial(self.WaitService, updated_serv.operation_id)
-      )
+      if updated_serv.operation_id is None or is_verbose:
+        getter = functools.partial(self.GetService, service_ref)
+      else:
+        getter = functools.partial(
+            self.WaitService,
+            updated_serv.operation_id,
+            service_ref,
+            release_track,
+        )
       poller = op_pollers.ServiceConditionPoller(
           getter, tracker, serv=updated_serv
       )
@@ -775,14 +803,16 @@ class ServerlessOperations(object):
       svc = service.Service(
           copy.deepcopy(prefetch.Message()), self.messages_module
       )
-      # if there's a template change and the name is set, this would fail
-      # unless we clear the name. Use the same forcing logic as down below.
-      # just to make sure there's no issue.
-      if config_changes_mod.AdjustsTemplate(config_changes):
-        if generate_name:
-          self._AddRevisionForcingChange(prefetch, validate_config_changes)
-        else:
-          validate_config_changes.append(_NewRevisionNonceChange(_Nonce()))
+    elif prefetch is not None:
+      # prefetch is None if we tried to get it and it's a create, or
+      # false if this is a replace so we never looked for it.
+      svc = self.GetService(service_ref)
+    # source deploy always has a template change so we should always
+    # force a new revision name.
+    if generate_name:
+      self._AddRevisionForcingChange(svc, validate_config_changes)
+    else:
+      validate_config_changes.append(_NewRevisionNonceChange(_Nonce()))
 
     tracker.StartStage(stages.VALIDATE_SERVICE)
     tracker.UpdateHeaderMessage('Validating Service...')
@@ -882,16 +912,14 @@ class ServerlessOperations(object):
        request to create/update the service.
     """
     requires_build = build_source is not None and not skip_build
-    should_validate_service = requires_build and release_track in [
-        base.ReleaseTrack.ALPHA,
-        base.ReleaseTrack.BETA,
-    ]
+
     region = build_region or self._region
+
     if tracker is None:
       tracker = progress_tracker.NoOpStagedProgressTracker(
           stages.ServiceStages(
               allow_unauthenticated is not None,
-              include_validate_service=should_validate_service,
+              include_validate_service=requires_build,
               include_upload_source=build_source is not None,
               include_build=requires_build,
               include_create_repo=repo_to_create is not None,
@@ -930,10 +958,9 @@ class ServerlessOperations(object):
     elif requires_build:
       new_conn = self._conn_context.GetContextWithRegionOverride(region)
       with new_conn:
-        if should_validate_service:
-          self._ValidateServiceBeforeSourceDeploy(
-              tracker, service_ref, prefetch, config_changes, generate_name
-          )
+        self._ValidateServiceBeforeSourceDeploy(
+            tracker, service_ref, prefetch, config_changes, generate_name
+        )
         # TODO(b/355762514): Either remove or re-enable this validation.
         # self._ValidateService(service_ref, config_changes)
         (
@@ -1029,11 +1056,15 @@ class ServerlessOperations(object):
     if not asyn and not dry_run:
       if updated_service.conditions.IsReady():
         return updated_service
-      getter = (
-          functools.partial(self.GetService, service_ref)
-          if updated_service.operation_id is None or is_verbose
-          else functools.partial(self.WaitService, updated_service.operation_id)
-      )
+      if updated_service.operation_id is None or is_verbose:
+        getter = functools.partial(self.GetService, service_ref)
+      else:
+        getter = functools.partial(
+            self.WaitService,
+            updated_service.operation_id,
+            service_ref,
+            release_track,
+        )
       poller = op_pollers.ServiceConditionPoller(
           getter,
           tracker,
@@ -1719,6 +1750,7 @@ class ServerlessOperations(object):
     create_request = messages.RunNamespacesJobsCreateRequest(
         job=new_job.Message(), parent=parent
     )
+    created_job = None
     with metrics.RecordDuration(metric_names.CREATE_JOB):
       try:
         created_job = job.Job(
@@ -1728,6 +1760,8 @@ class ServerlessOperations(object):
         raise serverless_exceptions.DeploymentFailedError(
             'Job [{}] already exists.'.format(job_ref.Name())
         )
+      except api_exceptions.HttpBadRequestError as e:
+        exceptions.reraise(serverless_exceptions.HttpError(e))
 
     if not asyn:
       getter = functools.partial(self.GetJob, job_ref)
@@ -1760,10 +1794,14 @@ class ServerlessOperations(object):
     replace_request = messages.RunNamespacesJobsReplaceJobRequest(
         job=update_job.Message(), name=job_ref.RelativeName()
     )
+    returned_job = None
     with metrics.RecordDuration(metric_names.UPDATE_JOB):
-      returned_job = job.Job(
-          self._client.namespaces_jobs.ReplaceJob(replace_request), messages
-      )
+      try:
+        returned_job = job.Job(
+            self._client.namespaces_jobs.ReplaceJob(replace_request), messages
+        )
+      except api_exceptions.HttpBadRequestError as e:
+        exceptions.reraise(serverless_exceptions.HttpError(e))
 
     if not asyn:
       getter = functools.partial(self.GetJob, job_ref)
@@ -2153,11 +2191,16 @@ class ServerlessOperations(object):
     for change in config_changes:
       run_job = change.Adjust(run_job)
 
-  def GetExecutionOverrides(self, tasks, task_timeout, container_overrides):
+  def GetExecutionOverrides(
+      self, tasks, task_timeout, priority_tier, container_overrides
+  ):
     return self.messages_module.Overrides(
         containerOverrides=container_overrides,
         taskCount=tasks,
         timeoutSeconds=task_timeout,
+        priorityTier=self.messages_module.Overrides.PriorityTierValueValuesEnum(
+            priority_tier.upper()
+        ),
     )
 
   def MakeContainerOverride(self, name, update_env_vars, args, clear_args):
