@@ -30,6 +30,7 @@ import subprocess
 import tempfile
 import textwrap
 
+from apitools.base.py import exceptions
 from googlecloudsdk.api_lib.oslogin import client as oslogin_client
 from googlecloudsdk.calliope import base
 from googlecloudsdk.command_lib.oslogin import oslogin_utils
@@ -578,14 +579,69 @@ class Keys(object):
           log.debug('Failed to create sentinel file: [{}]'.format(e))
 
 
-def CertFileFromInstance(project_id, zone, instance_id):
+def CertFileFromCloudRunDeployment(project, region, deployment):
+  cert_dir = os.path.realpath(files.ExpandHomeDir(CERTIFICATE_DIR))
+  return os.path.join(
+      cert_dir,
+      '{}_{}_{}-cert.pub'.format(project, region, deployment),
+  )
+
+
+def CertFileFromAppEngineInstance(project, service, version, instance):
+  cert_dir = os.path.realpath(files.ExpandHomeDir(CERTIFICATE_DIR))
+  return os.path.join(
+      cert_dir,
+      '{}_{}_{}_{}-cert.pub'.format(project, service, version, instance),
+  )
+
+
+def CertFileFromComputeInstance(project_id, zone, instance_id):
   cert_dir = os.path.realpath(files.ExpandHomeDir(CERTIFICATE_DIR))
   return os.path.join(
       cert_dir, '{}_{}_{}-cert.pub'.format(project_id, zone, instance_id)
   )
 
 
-def WriteCertificate(cert, project_id, zone, instance_id):
+def WriteCloudRunCertificate(cert, project, region, deployment):
+  """Writes a certificate associated with the key pair for Cloud Run.
+
+  Args:
+    cert: string, The SSH certificate data.
+    project: string, The project ID of the instance.
+    region: string, The region of the deployment.
+    deployment: string, The deployment name.
+  """
+  cert_dir = os.path.realpath(files.ExpandHomeDir(CERTIFICATE_DIR))
+  files.MakeDir(cert_dir, mode=0o700)
+
+  filepath = CertFileFromCloudRunDeployment(project, region, deployment)
+  try:
+    files.WriteFileContents(filepath, cert)
+  except files.Error as e:
+    log.debug('Failed to update the certificate {}: [{}]'.format(filepath, e))
+
+
+def WriteAppEngineCertificate(cert, project, service, version, instance):
+  """Writes a certificate associated with the key pair for App Engine.
+
+  Args:
+    cert: string, The SSH certificate data.
+    project: string, The project ID of the instance.
+    service: string, The service of the instance.
+    version: string, The version of the instance.
+    instance: string, The instance ID.
+  """
+  cert_dir = os.path.realpath(files.ExpandHomeDir(CERTIFICATE_DIR))
+  files.MakeDir(cert_dir, mode=0o700)
+
+  filepath = CertFileFromAppEngineInstance(project, service, version, instance)
+  try:
+    files.WriteFileContents(filepath, cert)
+  except files.Error as e:
+    log.debug('Failed to update the certificate {}: [{}]'.format(filepath, e))
+
+
+def WriteComputeCertificate(cert, project_id, zone, instance_id):
   """Writes a certificate associated with the key pair.
 
   Args:
@@ -597,7 +653,7 @@ def WriteCertificate(cert, project_id, zone, instance_id):
   cert_dir = os.path.realpath(files.ExpandHomeDir(CERTIFICATE_DIR))
   files.MakeDir(cert_dir, mode=0o700)
 
-  filepath = CertFileFromInstance(project_id, zone, instance_id)
+  filepath = CertFileFromComputeInstance(project_id, zone, instance_id)
   try:
     files.WriteFileContents(filepath, cert)
   except files.Error as e:
@@ -612,13 +668,18 @@ def DeleteCertificateFile(project_id, zone, instance_id):
     zone: string, The zone of the instance.
     instance_id: string, The instance ID.
   """
-  filepath = CertFileFromInstance(project_id, zone, instance_id)
+  filepath = CertFileFromComputeInstance(project_id, zone, instance_id)
   if os.path.exists(filepath):
     os.remove(filepath)
 
 
 def ValidateCertificate(
-    oslogin_state, project_id, zone, instance_id
+    oslogin_state,
+    project_id=None,
+    zone=None,
+    instance_id=None,
+    app_engine_params=None,
+    cloud_run_params=None,
 ):
   """Checks if the certificate is currently valid for a given instance.
 
@@ -627,6 +688,10 @@ def ValidateCertificate(
     project_id: string, The project ID of the instance.
     zone: string, The zone of the instance.
     instance_id: string, The instance ID.
+    app_engine_params: dict, values of (appsId, servicesId, versionId,
+      instanceId, serviceAccount, region) for App Engine instances.
+    cloud_run_params: dict, values of (region, deployment_name, project_id) for
+      Cloud Run deployments.
   """
   def IsCertValid(cert):
     time_format = '%Y-%m-%dT%H:%M:%S'
@@ -637,7 +702,22 @@ def ValidateCertificate(
     end = datetime.datetime.strptime(match[1], time_format)
     now = datetime.datetime.now()
     oslogin_state.signed_ssh_key = now > start and now < end
-  cert_file = CertFileFromInstance(project_id, zone, instance_id)
+
+  if app_engine_params:
+    cert_file = CertFileFromAppEngineInstance(
+        app_engine_params['appsId'],
+        app_engine_params['servicesId'],
+        app_engine_params['versionsId'],
+        app_engine_params['instancesId'],
+    )
+  elif cloud_run_params:
+    cert_file = CertFileFromCloudRunDeployment(
+        project_id,
+        cloud_run_params['region'],
+        cloud_run_params['deployment_name'],
+    )
+  else:
+    cert_file = CertFileFromComputeInstance(project_id, zone, instance_id)
   cmd = KeygenCommand(cert_file, print_cert=True)
   try:
     if os.path.exists(cert_file):
@@ -1063,6 +1143,82 @@ def _IsInstanceWindows(instance, messages=None):
   )
 
 
+def _SignAndWriteCloudRunCertificate(
+    oslogin,
+    public_key,
+    cloud_run_params,
+):
+  """Signs and writes a certificate for the given Cloud Run service."""
+  project_id = cloud_run_params['project_id']
+  region = cloud_run_params['region']
+  deployment_name = cloud_run_params['deployment_name']
+  sign_response = oslogin.SignSshPublicKeyForInstance(
+      public_key,
+      project_id,
+      region,
+      cloud_run_deployment=(
+          f'projects/{project_id}/locations/{region}/services/{deployment_name}'
+      ),
+  )
+  WriteCloudRunCertificate(
+      sign_response.signedSshPublicKey,
+      project_id,
+      region,
+      deployment_name,
+  )
+
+
+def _SignAndWriteAppEngineCertificate(
+    oslogin,
+    public_key,
+    project,
+    region,
+    app_engine_params,
+):
+  """Signs and writes a certificate for the given App Engine instance."""
+  sign_response = oslogin.SignSshPublicKeyForInstance(
+      public_key,
+      project.name,
+      region,
+      service_account=app_engine_params['serviceAccount'],
+      app_engine_instance=(
+          f'apps/{app_engine_params["appsId"]}/services/{app_engine_params["servicesId"]}/versions/{app_engine_params["versionsId"]}/instances/{app_engine_params["instancesId"]}'
+      ),
+  )
+  WriteAppEngineCertificate(
+      sign_response.signedSshPublicKey,
+      app_engine_params['appsId'],
+      app_engine_params['servicesId'],
+      app_engine_params['versionsId'],
+      app_engine_params['instancesId'],
+  )
+
+
+def _SignAndWriteComputeCertificate(
+    oslogin,
+    public_key,
+    project,
+    region,
+    zone,
+    instance,
+):
+  """Signs and writes a certificate for the given Compute instance."""
+  sign_response = oslogin.SignSshPublicKeyForInstance(
+      public_key,
+      project.name,
+      region,
+      service_account=instance.serviceAccounts[0].email
+      if instance.serviceAccounts
+      else '',
+      compute_instance=(
+          f'projects/{project.name}/zones/{zone}/instances/{instance.id}'
+      ),
+  )
+  WriteComputeCertificate(
+      sign_response.signedSshPublicKey, project.name, zone, instance.id
+  )
+
+
 def GetOsloginState(
     instance,
     project,
@@ -1070,6 +1226,8 @@ def GetOsloginState(
     public_key,
     expiration_time,
     release_track,
+    app_engine_params=None,
+    cloud_run_params=None,
     username_requested=False,
     instance_enable_oslogin=None,
     instance_enable_2fa=None,
@@ -1093,6 +1251,14 @@ def GetOsloginState(
       not be set to expire.  If not None, an existing key may be modified with
       the new expiry.
     release_track: release_track, The object representing the release track.
+    app_engine_params: dict, The fields required to identify an App Engine
+      instance. This should be None for Compute instances and Cloud Run
+      deployments. If present, this should contain fields for 'appsId',
+      'servicesId', 'versionsId', 'instancesId', and 'serviceAccount'.
+    cloud_run_params: dict, The fields required to identify a Cloud Run
+      deployment. This should be None for Compute and App Engine instances. If
+      present, this should contain fields for 'deployment_name', 'project_id',
+      and 'region'.
     username_requested: bool, True if the user has passed a specific username in
       the args.
     instance_enable_oslogin: True if the instance's metadata indicates that OS
@@ -1118,12 +1284,15 @@ def GetOsloginState(
   """
   oslogin_state = OsloginState(user=requested_user)
 
-  oslogin_enabled = FeatureEnabledInMetadata(
-      instance,
-      project,
-      OSLOGIN_ENABLE_METADATA_KEY,
-      instance_override=instance_enable_oslogin,
-  )
+  if cloud_run_params:
+    oslogin_enabled = True
+  else:
+    oslogin_enabled = FeatureEnabledInMetadata(
+        instance,
+        project,
+        OSLOGIN_ENABLE_METADATA_KEY,
+        instance_override=instance_enable_oslogin,
+    )
 
   if not oslogin_enabled:
     # OS Login is disabled by default.
@@ -1136,25 +1305,31 @@ def GetOsloginState(
     return oslogin_state
 
   oslogin_state.oslogin_enabled = oslogin_enabled
-  oslogin_state.oslogin_2fa_enabled = FeatureEnabledInMetadata(
-      instance,
-      project,
-      OSLOGIN_ENABLE_2FA_METADATA_KEY,
-      instance_override=instance_enable_2fa,
-  )
-  oslogin_state.security_keys_enabled = FeatureEnabledInMetadata(
-      instance,
-      project,
-      OSLOGIN_ENABLE_SK_METADATA_KEY,
-      instance_override=instance_enable_security_keys,
-  )
   oslogin_state.third_party_user = IsThirdPartyUser()
-  oslogin_state.require_certificates = FeatureEnabledInMetadata(
-      instance,
-      project,
-      OSLOGIN_ENABLE_CERTIFICATES_METADATA_KEY,
-      instance_override=instance_require_certificates,
-  )
+
+  if cloud_run_params:
+    oslogin_state.oslogin_2fa_enabled = False
+    oslogin_state.security_keys_enabled = False
+    oslogin_state.require_certificates = True
+  else:
+    oslogin_state.oslogin_2fa_enabled = FeatureEnabledInMetadata(
+        instance,
+        project,
+        OSLOGIN_ENABLE_2FA_METADATA_KEY,
+        instance_override=instance_enable_2fa,
+    )
+    oslogin_state.security_keys_enabled = FeatureEnabledInMetadata(
+        instance,
+        project,
+        OSLOGIN_ENABLE_SK_METADATA_KEY,
+        instance_override=instance_enable_security_keys,
+    )
+    oslogin_state.require_certificates = FeatureEnabledInMetadata(
+        instance,
+        project,
+        OSLOGIN_ENABLE_CERTIFICATES_METADATA_KEY,
+        instance_override=instance_require_certificates,
+    )
 
   env = Environment.Current()
   if env.suite == Suite.PUTTY:
@@ -1170,45 +1345,67 @@ def GetOsloginState(
       or properties.VALUES.core.account.Get()
   )
 
+  zone = None
+  region = None
   if instance and instance.zone:
     zone = instance.zone.split('/').pop()
     # Inclusively trim suffix from last '-' to convert a zone into a region.
     region = zone[: zone.rindex('-')]
-  else:
-    zone = None
-    region = None
+  elif app_engine_params:
+    region = app_engine_params['region']
 
-  if (
-      release_track
-      in [
-          base.ReleaseTrack.ALPHA,
-          base.ReleaseTrack.BETA,
-      ]
-      and (oslogin_state.third_party_user or oslogin_state.require_certificates)
-  ):
+  if release_track in [
+      base.ReleaseTrack.ALPHA,
+      base.ReleaseTrack.BETA,
+  ] and (oslogin_state.third_party_user or oslogin_state.require_certificates):
     user_email = quote(user_email, safe=':@')
-    ValidateCertificate(oslogin_state, project.name, zone, instance.id)
-    # TODO: b/395159475 - Conditionally call ProvisionPosixAccount based on if
-    # a regional GetLoginProfile call returns NOT_FOUND.
-    oslogin.ProvisionPosixAccount(user_email, project.name, region)
+    if app_engine_params:
+      ValidateCertificate(oslogin_state, app_engine_params)
+    elif cloud_run_params:
+      ValidateCertificate(
+          oslogin_state,
+          cloud_run_params=cloud_run_params,
+          project_id=cloud_run_params['project_id'],
+      )
+    else:
+      ValidateCertificate(oslogin_state, project.name, zone, instance.id)
     if not oslogin_state.signed_ssh_key:
-      sign_response = oslogin.SignSshPublicKeyForInstance(
-          public_key,
-          project.name,
-          region,
-          service_account=instance.serviceAccounts[0].email
-          if instance.serviceAccounts
-          else '',
-          compute_instance=(
-              f'projects/{project.name}/zones/{zone}/instances/{instance.id}'
-          ),
-      )
-      WriteCertificate(
-          sign_response.signedSshPublicKey, project.name, zone, instance.id
-      )
+      try:
+        if app_engine_params:
+          _SignAndWriteAppEngineCertificate(
+              oslogin, public_key, project, region, app_engine_params
+          )
+        elif cloud_run_params:
+          _SignAndWriteCloudRunCertificate(
+              oslogin,
+              public_key,
+              cloud_run_params,
+          )
+        else:
+          _SignAndWriteComputeCertificate(
+              oslogin, public_key, project, region, zone, instance
+          )
+      except exceptions.HttpNotFoundError:
+        log.status.Print(
+            'No OS Login profile found for user [{0}] for project [{1}].'
+            ' Creating POSIX account.'.format(user_email, project.name)
+        )
+        oslogin.ProvisionPosixAccount(user_email, project.name, region)
+        if app_engine_params:
+          _SignAndWriteAppEngineCertificate(
+              oslogin, public_key, project, region, app_engine_params
+          )
+        elif cloud_run_params:
+          _SignAndWriteCloudRunCertificate(
+              oslogin, public_key, cloud_run_params
+          )
+        else:
+          _SignAndWriteComputeCertificate(
+              oslogin, public_key, project, region, zone, instance
+          )
     login_profile = oslogin.GetLoginProfile(
         user_email,
-        project.name,
+        cloud_run_params['project_id'] if cloud_run_params else project.name,
         include_security_keys=oslogin_state.security_keys_enabled,
     )
   # Check to see if public key is already in profile and POSIX information
@@ -1229,7 +1426,7 @@ def GetOsloginState(
       )
     login_profile = oslogin.GetLoginProfile(
         user_email,
-        project.name,
+        cloud_run_params['project_id'] if cloud_run_params else project.name,
         include_security_keys=oslogin_state.security_keys_enabled,
     )
     if oslogin_state.security_keys_enabled:
@@ -1522,6 +1719,7 @@ def _BuildIapTunnelProxyCommandArgs(iap_tunnel_args, env):
 
   Raises:
     BadCharacterError: If instance arg contains any invalid characters.
+    ValueError: If no instance or cloud run args are provided.
 
   Returns:
     [str], the additional arguments for OpenSSH or Putty.
@@ -1532,12 +1730,13 @@ def _BuildIapTunnelProxyCommandArgs(iap_tunnel_args, env):
   # A relaxed validation of the passed in instance name / IP / hostname.
   # Mostly so potentially malicious names don't get passed in the resulting
   # generated command.
-  allowed_non_alnum_chars = {'-', '_', '.'}
-  for char in iap_tunnel_args.instance:
-    if not char.isalnum() and char not in allowed_non_alnum_chars:
-      raise BadCharacterError(
-          'Instance name/IP/hostname contains illegal characters.'
-      )
+  if iap_tunnel_args.instance:
+    allowed_non_alnum_chars = {'-', '_', '.'}
+    for char in iap_tunnel_args.instance:
+      if not char.isalnum() and char not in allowed_non_alnum_chars:
+        raise BadCharacterError(
+            'Instance name/IP/hostname contains illegal characters.'
+        )
 
   gcloud_command = execution_utils.ArgsForGcloud()
   # Applying _EscapeProxyCommandArg to the first item (the python executable
@@ -1554,23 +1753,48 @@ def _BuildIapTunnelProxyCommandArgs(iap_tunnel_args, env):
   port_token, quotation = (
       ('%port', '\"') if env.suite is Suite.PUTTY else ('%p', '\'')
   )
-  gcloud_command.extend([
-      'compute',
-      'start-iap-tunnel',
-      quotation + iap_tunnel_args.instance + quotation,
-      quotation + port_token + quotation,
-      '--listen-on-stdin',
-      '--project=' + iap_tunnel_args.project,
-  ])
-  if iap_tunnel_args.zone:
-    gcloud_command.append('--zone=' + iap_tunnel_args.zone)
-  if iap_tunnel_args.region:
-    gcloud_command.append('--region=' + iap_tunnel_args.region)
-  if iap_tunnel_args.network:
-    gcloud_command.append('--network=' + iap_tunnel_args.network)
+  if iap_tunnel_args.cloud_run_args:
+    gcloud_command.extend([
+        'run',
+        'start-iap-tunnel',
+        '--project_number='
+        + str(iap_tunnel_args.cloud_run_args['project_number']),
+        '--project_id='
+        + iap_tunnel_args.project,
+        '--workload_type='
+        + iap_tunnel_args.cloud_run_args['workload_type'].value,
+        '--deployment_name='
+        + iap_tunnel_args.cloud_run_args['deployment_name'],
+        '--region=' + iap_tunnel_args.region,
+    ])
+    if iap_tunnel_args.cloud_run_args['instance_id']:
+      gcloud_command.append(
+          '--instance=' + iap_tunnel_args.cloud_run_args['instance_id']
+      )
+    if iap_tunnel_args.cloud_run_args['container_id']:
+      gcloud_command.append(
+          '--container=' + iap_tunnel_args.cloud_run_args['container_id']
+      )
+  elif iap_tunnel_args.instance:
+    gcloud_command.extend([
+        'compute',
+        'start-iap-tunnel',
+        quotation + iap_tunnel_args.instance + quotation,
+        quotation + port_token + quotation,
+        '--listen-on-stdin',
+        '--project=' + iap_tunnel_args.project,
+    ])
+    if iap_tunnel_args.zone:
+      gcloud_command.append('--zone=' + iap_tunnel_args.zone)
+    if iap_tunnel_args.region:
+      gcloud_command.append('--region=' + iap_tunnel_args.region)
+    if iap_tunnel_args.network:
+      gcloud_command.append('--network=' + iap_tunnel_args.network)
+  else:
+    raise ValueError('No instance or cloud run args provided.')
+
   for arg in iap_tunnel_args.pass_through_args:
     gcloud_command.append(_EscapeProxyCommandArg(arg, env))
-
   verbosity = log.GetVerbosityName()
   if verbosity:
     gcloud_command.append('--verbosity=' + verbosity)
