@@ -15,6 +15,7 @@
 """Updates the settings of a Cloud SQL instance."""
 
 from __future__ import absolute_import
+from __future__ import annotations
 from __future__ import division
 from __future__ import unicode_literals
 
@@ -311,6 +312,7 @@ def AddBaseArgs(parser):
             'over IP.'))
   flags.AddStorageAutoIncrease(parser)
   flags.AddStorageSize(parser)
+  flags.AddStorageType(parser)
   flags.AddTier(parser, is_patch=True)
   flags.AddEdition(parser)
   flags.AddEnablePointInTimeRecovery(parser)
@@ -377,10 +379,14 @@ def AddBetaArgs(parser):
   labels_util.AddUpdateLabelsFlags(parser, enable_clear=True)
   flags.AddReplicationLagMaxSecondsForRecreate(parser)
   flags.AddReconcilePsaNetworking(parser)
-  flags.AddEnableAcceleratedReplicaMode(parser)
   unc_mappings_group = parser.add_mutually_exclusive_group(hidden=True)
   flags.AddUncMappings(unc_mappings_group)
   flags.AddClearUncMappings(unc_mappings_group)
+  flags.AddDataApiAccess(parser)
+  flags.AddServerCertificateRotationMode(parser)
+  flags.AddPerformanceCaptureConfig(parser, hidden=True)
+  flags.AddSqlServerEntraId(parser, hidden=True)
+  flags.AddClearEntraIdConfig(parser, hidden=True)
 
 
 def AddAlphaArgs(unused_parser):
@@ -465,6 +471,13 @@ def RunBasePatchCommand(args, release_track):
       sql_messages.SqlInstancesGetRequest(
           project=instance_ref.project, instance=instance_ref.instance))
 
+  if args.IsKnownAndSpecified(
+      'performance_capture_config'
+  ) and not original_instance_resource.databaseVersion.name.startswith('MYSQL'):
+    raise exceptions.ArgumentError(
+        '`--performance-capture-config` is only supported for MySQL instances.'
+    )
+
   if (
       args.IsSpecified('deny_maintenance_period_start_date')
       or args.IsSpecified('deny_maintenance_period_end_date')
@@ -516,14 +529,6 @@ def RunBasePatchCommand(args, release_track):
             ' arguments excluding gcloud wide flags'
         )
 
-  if args.IsKnownAndSpecified('enable_accelerated_replica_mode'):
-    if not api_util.InstancesV1Beta4.IsMysqlDatabaseVersion(
-        original_instance_resource.databaseVersion
-    ):
-      raise exceptions.ArgumentError(
-          '--enable-accelerated-replica-mode is only supported for MySQL.'
-      )
-
   patch_instance = command_util.InstancesV1Beta4.ConstructPatchInstanceFromArgs(
       sql_messages,
       args,
